using System.Linq.Expressions;
using ldap_cesi.Models;

namespace ldap_cesi.Services.Interfaces;

public interface IServiceBase<T, TDto, TCreateDto, TUpdateDto>
    where T : class
    where TDto : class
    where TCreateDto : class
    where TUpdateDto : class
{
    Task<IResponseDataModel<List<T>>> GetAll();
    Task<IResponseDataModel<T>> GetById(int id);
    Task<IResponseDataModel<TDto>> GetByIdWithRelations(int id, params Expression<Func<T, object>>[] relationsAInclures); // préciser avec une ou des fonctions lambda les relations à inclure dans la réponse
    Task<IResponseDataModel<List<TDto>>> GetAllWithRelationsAsync(params Expression<Func<T, object>>[] relationsAInclure);
    Task<IResponseDataModel<T>> Create(TCreateDto dto);
    Task<IResponseDataModel<T>> Update(TUpdateDto dto);
    Task<IResponseDataModel<string>> Delete(int id);
}